<?php 

/**
 * Formulaire pour créer une cellule
 * (des champs textes s'affichant dans un rectangle)
 */
if (!defined('_ECRIRE_INC_VERSION')) return;
 
 include_spip('inc/actions');
 include_spip('inc/editer');
      
/**
 * Déclaration des champs du formulaire
 *
 * Références des vérifications :
 * @link: https://contrib.spip.net/References-des-verifications
 */
function formulaires_editer_pdform_cellule_saisies_dist($id_pdform_cellule='new', $retour='', $associer_objet = '', $lier_trad=0, $config_fonc='', $row=array(), $hidden=''){
	$id_cellule = intval($id_pdform_cellule);
	
	include_spip('inc/filtres'); // fonctions pour les objets (necessaire en cas d'association)
	
	$saisies = array(
		array( // le champ éditable
			'saisie' => 'fieldset',
			'options' => array(
				'nom' => 'fieldset_champ_editable',
				'label' => _T('pdform_cellule:fieldset_champ_editable_label'),
				'explication' => _T('pdform_cellule:fieldset_champ_editable_explication'),
			),
			'saisies' => array( // les champs dans le fieldset 
				array(
				'saisie' => 'hidden', // l'identifiant unique de la cellule
				'options' => array(
					'nom' => 'id_pdform_cellule',
					'defaut' => $id_cellule,
					),
				),
				array(
					'saisie' => 'hidden', // l'association éventuelle
					'options' => array(
						'nom' => 'associer_objet',
						'defaut' => $associer_objet,
					),
				),
				array(
					'saisie' => 'tables_nourricieres', // l'objet dont le champ éditable provient
					'options' => array(
						'nom' => 'table_nourriciere',
						'label' => _T('pdform_cellule:champ_table_nourriciere_label'),
						'explication' => _T('pdform_cellule:champ_table_nourriciere_explication'),
						'afficher_icone' => 'oui',
						#'select' => 'oui',
					),
				),
				array(
					'saisie' => 'input',
					'options' => array(
						'nom' => 'titre',
						'label' => _T('pdform_cellule:champ_titre_label'),
						'explication' => _T('pdform_cellule:champ_titre_explication'),
					),
				),
				'verifier' => array( // le nombre est de type float
					'type' => 'taille',
					'options' => array(
						'maxi' => '255',
						'min' => '0',
					),
				),
				array(
					'saisie' => 'textarea',
					'options' => array(
						'nom' => 'texte',
						'label' => _T('pdform_cellule:champ_celltxt_label'),
						'explication' => _T('pdform_cellule:champ_celltxt_explication'),
					),
				),
			),
		),
		array( // le fieldset cell
			'saisie' => 'fieldset',
			'options' => array(
				'nom' => 'fieldset_cell',
				'label' => _T('pdform_cellule:fieldset_cell_label'),
				'explication' => _T('pdform_cellule:fieldset_cell_explication'),
			),
			'saisies' => array( // les champs dans le fieldset 
				array(
					'saisie' => 'pdform_polices',
					'options' => array(
						'nom' => 'police',
						'label' => _T('pdform_patron:champ_police_label'),
						'aide' => 'pdformfontes/polices',
					),
				),
				array(
					'saisie' => 'input',
					'options' => array(
						'nom' => 'taille',
						'label' => _T('pdform_patron:champ_taille_label'),
						'aide' => 'pdformfontes/tailles',
					),
				),
				array(
					'saisie' => 'selection',
					'options' => array(
						'nom' => 'style',
						'label' => _T('pdform_patron:champ_style_label'),
						'aide' => 'pdformfontes/styles',
						'cacher_option_intro' => 'oui',
						'data' => array(
							'' => _T('pdform_patron:data_fontstyle_'),
							'B' => _T('pdform_patron:data_fontstyle_b'),
							'U' => _T('pdform_patron:data_fontstyle_u'),
							'I' => _T('pdform_patron:data_fontstyle_i'),
						),
					),
				),
				array( // couleur d'affichage de la police
					'saisie' => 'couleur',
					'options' => array(
						'nom' => 'couleur_texte',
						'label' => _T('pdform_patron:champ_couleur_texte_label'),
						'explication' => _T('pdform_patron:champ_couleur_texte_explication'),
					),
				),
				'verifier' => array( // la valeur est une couleur en hexa 
					'type' => 'couleur',
					'options' => array(
						'type' => 'hexa',
						'normaliser' => 'oui'
					),
				),
				array(
					'saisie' => 'input',
					'options' => array(
						'nom' => 'x',
						'label' => _T('pdform_patron:champ_x_label'),
						'explication' => _T('pdform_patron:champ_x_explication'),
					),
				),
				'verifier' => array( // le nombre est de type float
					'type' => 'decimal',
					'options' => array(
						'maxi' => '1000',
						'min' => '0',
						'nb_decimales' => '2',
						'normaliser' => 'oui',
						'separateur' => '.',
					),
				),
				array(
					'saisie' => 'input',
					'options' => array(
						'nom' => 'y',
						'label' => _T('pdform_patron:champ_y_label'),
						'explication' => _T('pdform_patron:champ_y_explication'),
					),
				),
				'verifier' => array( // le nombre est de type float
					'type' => 'decimal',
					'options' => array(
						'maxi' => '1000',
						'min' => '0',
						'nb_decimales' => '2',
						'normaliser' => 'oui',
						'separateur' => '.',
					),
				),
				array(
					'saisie' => 'input',
					'options' => array(
						'nom' => 'largeur',
						'label' => _T('pdform_patron:champ_largeur_label'),
						'explication' => _T('pdform_patron:champ_largeur_explication'),
					),
				),
				'verifier' => array( // le nombre est de type float
					'type' => 'decimal',
					'options' => array(
						'maxi' => '1000',
						'min' => '0',
						'nb_decimales' => '2',
						'normaliser' => 'oui',
						'separateur' => '.',
					),
				),
				array(
					'saisie' => 'input',
					'options' => array(
						'nom' => 'hauteur',
						'label' => _T('pdform_patron:champ_hauteur_label'),
						'explication' => _T('pdform_patron:champ_hauteur_explication'),
						'defaut' => '5',
					),
				),
				'verifier' => array( // le nombre est de type float
					'type' => 'decimal',
					'options' => array(
						'maxi' => '1000',
						'min' => '0',
						'nb_decimales' => '2',
						'normaliser' => 'oui',
						'separateur' => '.',
					),
				),
				array(
					'saisie' => 'selection',
					'options' => array(
						'nom' => 'bordure',
						'label' => _T('pdform_patron:champ_bordure_label'),
						'explication' => _T('pdform_patron:champ_bordure_explication'),
						'cacher_option_intro' => 'oui',
						#'multiple' => 'non',
						'data' => array(
							'0' => _T('pdform_patron:data_border_0'), // aucun bord tracé
							'1' => _T('pdform_patron:data_border_1'), // un cadre doit être tracé
							'L' => _T('pdform_patron:data_border_l'), // un bord gauche doit être tracé
							'T' => _T('pdform_patron:data_border_t'), // un bord haut doit être tracé
							'R' => _T('pdform_patron:data_border_r'), // un bord droit doit être tracé
							'B' => _T('pdform_patron:data_border_b'), // un bord bas doit être tracé
						),
					),
				),
				'verifier' => array( // les valeurs peuvent être 0|1|L|T|R|B
					'type' => 'regex',
					'options' => array(
						'modele' => '@^[0-1L|T|R|B]$@',
						'message_erreur' => _T('pdform_patron:message_erreur_regex_border'),
					),
				),
				array( // Fixe la couleur des bords de la cellule
					'saisie' => 'couleur',
					'options' => array(
						'nom' => 'couleur_trace',
						'label' => _T('pdform_patron:champ_couleur_trace_label'),
						'explication' => _T('pdform_cellule:champ_couleur_trace_explication'),
						'afficher_si' => '@bordure@ != 0',
					),
				),
				'verifier' => array( // tous les types d'url
					'type' => 'couleur',
					'options' => array(
						'type' => 'hexa',
						'normaliser' => 'oui'
					),
				),
				array(
					'saisie' => 'case',
					'options' => array(
						'nom' => 'plein',
						'label' => _T('pdform_patron:champ_rempli_label'),
						'explication' => _T('pdform_patron:champ_rempli_explication'),
						'valeur_oui' => '1',
						'valeur_non' => '0',
					),
				),
					array( // Fixe la couleur de remplissage du fond de la cellule
					'saisie' => 'couleur',
					'options' => array(
						'nom' => 'couleur_rempli',
						'label' => _T('pdform_patron:champ_couleur_rempli_label'),
						'explication' => _T('pdform_patron:champ_couleur_rempli_explication'),
						'afficher_si' => '@plein@ == 1',
					),
				),
				'verifier' => array( // tous les types d'url
					'type' => 'couleur',
					'options' => array(
						'type' => 'hexa',
						'normaliser' => 'oui'
					),
				),
				array(
					'saisie' => 'selection',
					'options' => array(
						'nom' => 'apres',
						'label' => _T('pdform_patron:champ_apres_label'),
						'explication' => _T('pdform_patron:champ_apres_explication'),
						'cacher_option_intro' => 'oui',
						'defaut' => '2', // en dessous (utile si cell avec des retours à la ligne)
						'data' => array(
							'0' => _T('pdform_patron:data_ln_0'),
							'1' => _T('pdform_patron:data_ln_1'),
							'2' => _T('pdform_patron:data_ln_2'),
						),
					),
				),
				'verifier' => array( // les valeurs peuvent être 0|1|2
					'type' => 'regex',
					'options' => array(
						'modele' => '@^[0-2]$@',
						'message_erreur' => _T('pdform_patron:message_erreur_regex_apres'),
					),
				),
				array(
					'saisie' => 'selection',
					'options' => array(
						'nom' => 'align',
						'label' => _T('pdform_patron:champ_align_label'),
						'explication' => _T('pdform_patron:champ_align_explication'),
						'cacher_option_intro' => 'oui',
						#'multiple' => 'non',
						'data' => array(
							'L' => _T('pdform_patron:data_align_l'),
							'C' => _T('pdform_patron:data_align_c'),
							'R' => _T('pdform_patron:data_align_r'),
						),
					),
				),
				'verifier' => array( // la valeur peuvent être L|C|R
					'type' => 'regex',
					'options' => array(
						'modele' => '@^[L|C|R]$@',
						'message_erreur' => _T('pdform_patron:message_erreur_regex_align'),
					),
				),
				array(
					'saisie' => 'input',
					'options' => array(
						'nom' => 'lien',
						'label' => _T('pdform_patron:champ_lien_label'),
						'explication' => _T('pdform_patron:champ_lien_explication'),
					),
				),
				'verifier' => array( // tous les types d'url
					'type' => 'url',
				),
			),
		),
		array( // le fieldset d'association
			'saisie' => 'fieldset',
			'options' => array(
				'afficher_si'=> '@associer_objet@ != ""',
				'nom' => 'association',
				'label' => _T('pdform_cellule:fieldset_association'),
				#'icone' => $icone_objet,
			),
			'saisies' => array( // les champs dans le fieldset 
				array(
					'saisie' => 'case',
					'options' => array(
						'nom' => 'c_associe',
						'label' => _T('pdform_cellule:texte_associer_cellule_a', array('str' => _T(strstr($associer_objet,'|',true).':titre_'.strstr($associer_objet,'|',true)), 'nb' => substr(strstr(_request('associer_objet'),'|'),1))),
						'valeur_oui' => 'oui', 
						'valeur_non' => 'non',
						'defaut' => 'oui',
					)
				),
			)
		)
	);

	return $saisies;
}

/**
  * Identifier le formulaire en faisant abstraction des parametres qui ne representent pas l'objet edite
  */
function formulaires_editer_pdform_cellule_identifier_dist($id_pdform_cellule='new', $retour='', $associer_objet = '', $lier_trad=0, $config_fonc='', $row=array(), $hidden=''){
	return serialize(array(intval($id_pdform_cellule), $associer_objet));
}
 
/**
  * Declarer les champs postes et y integrer les valeurs par defaut
  */
function formulaires_editer_pdform_cellule_charger_dist($id_pdform_cellule='new', $retour='', $associer_objet = '', $lier_trad=0, $config_fonc='', $row=array(), $hidden=''){
	// Initialisation des valeurs à transmettre au formulaire
 	$valeurs = array();

	$valeurs = formulaires_editer_objet_charger('pdform_cellule',$id_pdform_cellule,'',$lier_trad,$retour,$config_fonc,$row,$hidden);

	# une cellule doit avoir une table_nourriciere

	# en création l'information doit être transmise par l'url
	if (!intval($id_pdform_cellule) and !$valeurs['table_nourriciere']){
		$valeurs['table_nourriciere'] = _request('table_nourriciere');
	}
	
	# Une association 
	$valeurs['associer_objet'] = $associer_objet ? $associer_objet : _request('associer_objet');
	if ($valeurs['associer_objet']) {
		$valeurs['c_associer'] = 'oui';
	}
	
	# normaliser en hexa le code rgb
	$couleurs = array(
		'couleur_trace' => $valeurs['couleur_trace'],
		'couleur_rempli' => $valeurs['couleur_rempli'],
		'couleur_texte' => $valeurs['couleur_texte'],
	);
	$pdfrom_couleurs = charger_fonction('pdform_couleurs', 'inc');
	foreach ($couleurs as $colonne => $rgb) {
		if ($rgb){
			$hexa = $pdfrom_couleurs($rgb);
			if ($hexa){
				$valeurs[$colonne] = $hexa;
			}
		}
	}
	return $valeurs;
}

/**
  * Verifier les champs postes et signaler d'eventuelles erreurs
  */
 function formulaires_editer_pdform_cellule_verifier_dist($id_pdform_cellule='new', $retour='', $associer_objet = '', $lier_trad=0, $config_fonc='', $row=array(), $hidden=''){
	$erreurs = array();//formulaires_editer_objet_verifier('pdform_cellule',$id_pdform_cellule);

	return $erreurs;
}

/**
  * Traiter les champs postes
  */
function formulaires_editer_pdform_cellule_traiter_dist($id_pdform_cellule='new', $retour='', $associer_objet = '', $lier_trad=0, $config_fonc='', $row=array(), $hidden=''){

	# normaliser en rgb le code en hexa
	$couleurs = array(
		'couleur_trace' => _request('couleur_trace'),
		'couleur_rempli' => _request('couleur_rempli'),
		'couleur_texte' => _request('couleur_texte'),
	);
	$pdfrom_couleurs = charger_fonction('pdform_couleurs', 'inc');
	foreach ($couleurs as $colonne => $hexa) {
		$rgb = $pdfrom_couleurs($hexa);
		if ($rgb){
			set_request($colonne, $rgb);
		}
	}

	$associer_objet = _request('associer_objet');
	// rompre l'association si nécessaire
	if (_request('c_associe') == 'non'){
		$associer_objet = '';
	}

	$retours = formulaires_editer_objet_traiter('pdform_cellule',$id_pdform_cellule,'',$lier_trad,$retour,$config_fonc,$row,$hidden);

	// Un lien a prendre en compte ?
 	if ($associer_objet and $id_pdform_cellule = $retours['id_pdform_cellule']) {
 		list($objet, $id_objet) = explode('|', $associer_objet);
 		if ($objet and $id_objet == intval($id_objet)) {
 			include_spip('action/editer_liens');
 			objet_associer(array('pdform_cellule' => $id_pdform_cellule), array($objet => $id_objet));
			// rediriger vers l'objet
			if (isset($redirect)) {
				$retours['redirect'] = parametre_url($redirect, 'id_lien_ajoute', $id_pdform_cellule, '&');
			}
 		}
 	}
	return $retours;

}

